;
; ** $VER: narrator.h 1.7 (12.3.91)
; ** Includes Release 40.15
; **
; ** V37 Narrator device C language include file
; **
; ** Copyright 1990, 1991 Joseph Katz/Mark Barton.
; ** All rights reserved.
; **
; ** This include file (narrator.h) may be freely distributed
; ** as long as the above copyright notice remains intact.
; **
;


IncludePath   "PureInclude:"
XIncludeFile "exec/io.pb"


  ;      Device Options

#NDB_NEWIORB  = 0 ;  Use new extended IORB
#NDB_WORDSYNC = 1 ;  Generate word sync messages
#NDB_SYLSYNC  = 2 ;  Generate syllable sync messages


#NDF_NEWIORB  = (1  <<  #NDB_NEWIORB)
#NDF_WORDSYNC = (1  <<  #NDB_WORDSYNC)
#NDF_SYLSYNC  = (1  <<  #NDB_SYLSYNC)



  ;      Error Codes

#ND_NoMem = -2 ;  Can't allocate memory
#ND_NoAudLib = -3 ;  Can't open audio device
#ND_MakeBad = -4 ;  Error in MakeLibrary call
#ND_UnitErr = -5 ;  Unit other than 0
#ND_CantAlloc = -6 ;  Can't allocate audio channel(s)
#ND_Unimpl = -7 ;  Unimplemented command
#ND_NoWrite = -8 ;  Read for mouth without write first
#ND_Expunged = -9 ;  Can't open, deferred expunge bit set
#ND_PhonErr     = -20 ;  Phoneme code spelling error
#ND_RateErr     = -21 ;  Rate out of bounds
#ND_PitchErr    = -22 ;  Pitch out of bounds
#ND_SexErr      = -23 ;  Sex not valid
#ND_ModeErr     = -24 ;  Mode not valid
#ND_FreqErr     = -25 ;  Sampling frequency out of bounds
#ND_VolErr      = -26 ;  Volume out of bounds
#ND_DCentErr    = -27 ;  Degree of centralization out of bounds
#ND_CentPhonErr = -28 ;  Invalid central phon



  ;  Input parameters and defaults

#DEFPITCH    = 110  ;  Default pitch
#DEFRATE     = 150  ;  Default speaking rate (wpm)
#DEFVOL     = 64  ;  Default volume (full)
#DEFFREQ     = 22200 ;  Default sampling frequency (Hz)
#MALE     = 0  ;  Male vocal tract
#FEMALE     = 1  ;  Female vocal tract
#NATURALF0   = 0  ;  Natural pitch contours
#ROBOTICF0   = 1  ;  Monotone pitch
#MANUALF0    = 2  ;  Manual setting of pitch contours
#DEFSEX     = #MALE ;  Default sex
#DEFMODE     = #NATURALF0 ;  Default mode
#DEFARTIC    = 100  ;  100% articulation (normal)
#DEFCENTRAL  = 0  ;  No centralization
#DEFF0PERT   = 0  ;  No F0 Perturbation
#DEFF0ENTHUS = 32  ;  Default F0 enthusiasm (in 32nds)
#DEFPRIORITY = 100  ;  Default speaking priority


   ;  Parameter bounds

#MINRATE     = 40  ;  Minimum speaking rate
#MAXRATE     = 400  ;  Maximum speaking rate
#MINPITCH    = 65  ;  Minimum pitch
#MAXPITCH    = 320  ;  Maximum pitch
#MINFREQ     = 5000 ;  Minimum sampling frequency
#MAXFREQ     = 28000 ;  Maximum sampling frequency
#MINVOL     = 0  ;  Minimum volume
#MAXVOL     = 64  ;  Maximum volume
#MINCENT      = 0  ;  Minimum degree of centralization
#MAXCENT    = 100  ;  Maximum degree of centralization


  ;     Standard Write request

Structure narrator_rb
 message.IOStdReq ;  Standard IORB
 rate.w   ;  Speaking rate (words/minute)
 pitch.w   ;  Baseline pitch in Hertz
 mode.w   ;  Pitch mode
 sex.w   ;  Sex of voice
 *ch_masks.b  ;  Pointer to audio alloc maps
 nm_masks.w  ;  Number of audio alloc maps
 volume.w   ;  Volume. 0 (off) thru 64
 sampfreq.w  ;  Audio sampling freq
 mouths.b   ;  If non-zero, generate mouths
 chanmask.b  ;  Which ch mask used (internal)
 numchan.b  ;  Num ch masks used (internal)
 flags.b   ;  New feature flags
 F0enthusiasm.b  ;  F0 excursion factor
 F0perturb.b  ;  Amount of F0 perturbation
 F1adj.b   ;  F1 adjustment in 15% steps
 F2adj.b    ;  F2 adjustment in 15% steps
 F3adj.b   ;  F3 adjustment in 15% steps
 A1adj.b   ;  A1 adjustment in decibels
 A2adj.b   ;  A2 adjustment in decibels
 A3adj.b   ;  A3 adjustment in decibels
 articulate.b  ;  Transition time multiplier
 centralize.b  ;  Degree of vowel centralization
 *centphon.b  ;  Pointer to central ASCII phon
 AVbias.b   ;  AV bias
 AFbias.b   ;  AF bias
 priority.b  ;  Priority while speaking
 pad1.b   ;  For alignment
EndStructure



  ;     Standard Read request

Structure mouth_rb
 voice.narrator_rb ;  Speech IORB
 width.b   ;  Width (returned value)
 height.b   ;  Height (returned value)
 shape.b   ;  Internal use, do not modify
 sync.b   ;  Returned sync events
EndStructure



