;
; ** $VER: text.h 39.0 (21.8.91)
; ** Includes Release 40.15
; **
; ** graphics library text structures
; **
; ** (C) Copyright 1985-1993 Commodore-Amiga, Inc.
; **     All Rights Reserved
;

IncludePath   "PureInclude:"
XIncludeFile "exec/ports.pb"
XIncludeFile "graphics/gfx.pb"
XIncludeFile "utility/tagitem.pb"

; ------ Font Styles ------------------------------------------------
#FS_NORMAL = 0 ;  normal text (no style bits set)
#FSB_UNDERLINED = 0 ;  underlined (under baseline)
#FSF_UNDERLINED = $01
#FSB_BOLD = 1 ;  bold face text (ORed w/ shifted)
#FSF_BOLD = $02
#FSB_ITALIC = 2 ;  italic (slanted 1:2 right)
#FSF_ITALIC = $04
#FSB_EXTENDED = 3 ;  extended face (wider than normal)
#FSF_EXTENDED = $08

#FSB_COLORFONT = 6 ;  this uses ColorTextFont structure
#FSF_COLORFONT = $40
#FSB_TAGGED = 7 ;  the TextAttr is really an TTextAttr,
#FSF_TAGGED = $80

; ------ Font Flags -------------------------------------------------
#FPB_ROMFONT = 0 ;  font is in rom
#FPF_ROMFONT = $01
#FPB_DISKFONT = 1 ;  font is from diskfont.library
#FPF_DISKFONT = $02
#FPB_REVPATH = 2 ;  designed path is reversed (e.g. left)
#FPF_REVPATH = $04
#FPB_TALLDOT = 3 ;  designed for hires non-interlaced
#FPF_TALLDOT = $08
#FPB_WIDEDOT = 4 ;  designed for lores interlaced
#FPF_WIDEDOT = $10
#FPB_PROPORTIONAL = 5 ;  character sizes can vary from nominal
#FPF_PROPORTIONAL = $20
#FPB_DESIGNED = 6 ;  size explicitly designed, not constructed
    ;  note: if you do not set this bit in your
    ;  textattr, then a font may be constructed
    ;  for you by scaling an existing rom or disk
    ;  font (under V36 and above).
#FPF_DESIGNED = $40
    ;  bit 7 is always clear for fonts on the graphics font list
#FPB_REMOVED = 7 ;  the font has been removed
#FPF_REMOVED = (1 << 7)

; ***** TextAttr node, matches text attributes in RastPort *********
Structure TextAttr
    *ta_Name.b  ;  name of the font
    ta_YSize.w  ;  height of the font
    ta_Style.b  ;  intrinsic font style
    ta_Flags.b  ;  font preferences and flags
EndStructure

Structure TTextAttr
    *tta_Name.b  ;  name of the font
    tta_YSize.w  ;  height of the font
    tta_Style.b  ;  intrinsic font style
    tta_Flags.b  ;  font preferences and flags
    *tta_Tags.TagItem ;  extended attributes
EndStructure


; ***** Text Tags **************************************************
#TA_DeviceDPI = (1 | #TAG_USER) ;  Tag value is Point union:
     ;  Hi word XDPI, Lo word YDPI

#MAXFONTMATCHWEIGHT = 32767 ;  perfect match from WeighTAMatch


; ***** TextFonts node *********************************************
Structure TextFont
    tf_Message.Message ;  reply message for font removal
    ;  font name in LN   \    used in this
    tf_YSize.w  ;  font height    |    order to best
    tf_Style.b  ;  font style    |    match a font
    tf_Flags.b  ;  preferences and flags  /    request.
    tf_XSize.w  ;  nominal font width
    tf_Baseline.w ;  distance from the top of char to baseline
    tf_BoldSmear.w ;  smear to affect a bold enhancement

    tf_Accessors.w ;  access count

    tf_LoChar.b  ;  the first character described here
    tf_HiChar.b  ;  the last character described here
    *tf_CharData.l ;  the bit character data

    tf_Modulo.w  ;  the row modulo for the strike font data
    *tf_CharLoc.l  ;  ptr to location data for the strike font
    ;    2 words: bit offset then size
    *tf_CharSpace.l ;  ptr to words of proportional spacing data
    *tf_CharKern.l ;  ptr to words of kerning data
EndStructure

;  unfortunately, this needs to be explicitly typed
;#tf_Extension = tf_Message\mn_ReplyPort

; ----- tfe_Flags0 (partial definition) ----------------------------
#TE0B_NOREMFONT = 0 ;  disallow RemFont for this font
#TE0F_NOREMFONT = $01

Structure TextFontExtension  ;  this structure is read-only
    tfe_MatchWord.w  ;  a magic cookie for the extension
    tfe_Flags0.b   ;  (system private flags)
    tfe_Flags1.b   ;  (system private flags)
    *tfe_BackPtr.TextFont ;  validation of compilation
    *tfe_OrigReplyPort.MsgPort ;  original value in tf_Extension
    *tfe_Tags.TagItem  ;  Text Tags for the font
    *tfe_OFontPatchS.w  ;  (system private use)
    *tfe_OFontPatchK.w  ;  (system private use)
    ;  this space is reserved for future expansion
EndStructure

; ***** ColorTextFont node *****************************************
; ----- ctf_Flags --------------------------------------------------
#CT_COLORMASK = $000F ;  mask to get to following color styles
#CT_COLORFONT = $0001 ;  color map contains designer's colors
#CT_GREYFONT = $0002 ;  color map describes even-stepped
    ;  brightnesses from low to high
#CT_ANTIALIAS = $0004 ;  zero background thru fully saturated char

#CTB_MAPCOLOR = 0 ;  map ctf_FgColor to the rp_FgPen if it's
#CTF_MAPCOLOR = $0001 ;  is a valid color within ctf_Low..ctf_High

; ----- ColorFontColors --------------------------------------------
Structure ColorFontColors
    cfc_Reserved.w ;  *must* be zero
    cfc_Count.w  ;  number of entries in cfc_ColorTable
    *cfc_ColorTable.w ;  4 bit per component color map packed xRGB
EndStructure

; ----- ColorTextFont ----------------------------------------------
Structure ColorTextFont
    ctf_TF.TextFont
    ctf_Flags.w  ;  extended flags
    ctf_Depth.b  ;  number of bit planes
    ctf_FgColor.b ;  color that is remapped to FgPen
    ctf_Low.b  ;  lowest color represented here
    ctf_High.b  ;  highest color represented here
    ctf_PlanePick.b ;  PlanePick ala Images
    ctf_PlaneOnOff.b ;  PlaneOnOff ala Images
    *ctf_ColorFontColors.ColorFontColors ;  colors for font
    *ctf_CharData.l[8] ; pointers to bit planes ala tf_CharData
EndStructure

; ***** TextExtent node ********************************************
Structure TextExtent
    te_Width.w  ;  same as TextLength
    te_Height.w  ;  same as tf_YSize
    te_Extent.Rectangle ;  relative to CP
EndStructure

