;
; ** $VER: view.h 39.34 (31.5.93)
; ** Includes Release 40.15
; **
; ** graphics view/viewport definintions
; **
; ** (C) Copyright 1985-1993 Commodore-Amiga, Inc.
; **     All Rights Reserved
;

IncludePath   "PureInclude:"
XIncludeFile "exec/types.pb"
XIncludeFile "exec/semaphores.pb"
XIncludeFile "graphics/gfx.pb"
XIncludeFile "graphics/copper.pb"
XIncludeFile "graphics/gfxnodes.pb"
XIncludeFile "graphics/monitor.pb"
XIncludeFile "graphics/displayinfo.pb"
XIncludeFile "hardware/custom.pb"

Structure ViewPort

  *Next.ViewPort
 *ColorMap.ColorMap ;  table of colors for this viewport
     ;  if this is nil, MakeVPort assumes default values
 *DspIns.CopList ;  used by MakeVPort()
 *SprIns.CopList ;  used by sprite stuff
 *ClrIns.CopList ;  used by sprite stuff
 *UCopIns.UCopList ;  User copper list
 DWidth.w
 DHeight.w
 DxOffset.w
 DyOffset.w
 Modes.w
 SpritePriorities.b
 ExtendedModes.b
 *RasInfo.RasInfo
EndStructure

Structure View
 *ViewPort.ViewPort
 *LOFCprList.cprlist   ;  used for interlaced and noninterlaced
 *SHFCprList.cprlist   ;  only used during interlace
 DyOffset.w
 DxOffset.w   ;  for complete View positioning
       ;  offsets are +- adjustments to standard #s
 Modes.w     ;  such as INTERLACE, GENLOC
EndStructure

;  these structures are obtained via GfxNew
;  and disposed by GfxFree
Structure ViewExtra
 n.ExtendedNode
 *View.View  ;  backwards link
 *Monitor.MonitorSpec ;  monitors for this view
 TopLine.w
EndStructure

;  this structure is obtained via GfxNew
;  and disposed by GfxFree
Structure ViewPortExtra
 n.ExtendedNode
 *ViewPort.ViewPort ;  backwards link
 DisplayClip.Rectangle ;  MakeVPort display clipping information
 ;  These are added for V39
 *VecTable.l  ;  Private
 *DriverData.l[2]
 Flags.w
 Origin.Point[2]  ;  First visible point relative to the DClip.
;       * One for each possible playfield.
;
 cop1ptr.l   ;  private
 cop2ptr.l   ;  private
EndStructure

;  All these VPXF_ flags are private
#VPXB_FREE_ME  = 0
#VPXF_FREE_ME  = (1   <<   #VPXB_FREE_ME)
#VPXB_LAST  = 1
#VPXF_LAST  = (1   <<   #VPXB_LAST)
#VPXB_STRADDLES_256 = 4
#VPXF_STRADDLES_256 = (1   <<   #VPXB_STRADDLES_256)
#VPXB_STRADDLES_512 = 5
#VPXF_STRADDLES_512 = (1   <<   #VPXB_STRADDLES_512)


#EXTEND_VSTRUCT = $1000 ;  unused bit in Modes field of View

#VPF_A2024       = $40 ;  VP?_ fields internal only
#VPF_TENHZ       = $20
#VPB_A2024       = 6
#VPB_TENHZ       = 4

;  defines used for Modes in IVPargs

#GENLOCK_VIDEO = $0002
#LACE  = $0004
#DOUBLESCAN = $0008
#SUPERHIRES = $0020
#PFBA  = $0040
#EXTRA_HALFBRITE = $0080
#GENLOCK_AUDIO = $0100
#DUALPF  = $0400
#HAM  = $0800
#EXTENDED_MODE = $1000
#VP_HIDE = $2000
#SPRITES = $4000
#HIRES  = $8000

Structure RasInfo ;  used by callers to and InitDspC()

    *Next.RasInfo     ;  used for dualpf
   *BitMap.BitMap
   RxOffset.w
   RyOffset.w    ;  scroll offsets in this BitMap
EndStructure

Structure ColorMap
 Flags.b
 Type.b
 Count.w
 *ColorTable.l
 *cm_vpe.ViewPortExtra
 *LowColorBits.l
 TransparencyPlane.b
 SpriteResolution.b
 SpriteResDefault.b ;  what resolution you get when you have set SPRITERESN_DEFAULT
 AuxFlags.b
 *cm_vp.ViewPort
 *NormalDisplayInfo.l
 *CoerceDisplayInfo.l
 *cm_batch_items.TagItem
 VPModeID.l
 *PalExtra.PaletteExtra
 SpriteBase_Even.w
 SpriteBase_Odd.w
 Bp_0_base.w
 Bp_1_base.w

EndStructure

;  if Type == 0 then ColorMap is V1.2/V1.3 compatible
;  if Type != 0 then ColorMap is V38    compatible
;  the system will never create other than V39 type colormaps when running V39

#COLORMAP_TYPE_V1_2 = $00
#COLORMAP_TYPE_V1_4 = $01
#COLORMAP_TYPE_V36 = #COLORMAP_TYPE_V1_4 ;  use this definition
#COLORMAP_TYPE_V39 = $02

;  Flags variable
#COLORMAP_TRANSPARENCY = $01
#COLORPLANE_TRANSPARENCY = $02
#BORDER_BLANKING  = $04
#BORDER_NOTRANSPARENCY = $08
#VIDEOCONTROL_BATCH = $10
#USER_COPPER_CLIP = $20
#BORDERSPRITES = $40

#CMF_CMTRANS = 0
#CMF_CPTRANS = 1
#CMF_BRDRBLNK = 2
#CMF_BRDNTRAN = 3
#CMF_BRDRSPRT = 6

#SPRITERESN_ECS  = 0
;  ^140ns, except in 35ns viewport, where it is 70ns.
#SPRITERESN_140NS = 1
#SPRITERESN_70NS  = 2
#SPRITERESN_35NS  = 3
#SPRITERESN_DEFAULT = -1

;  AuxFlags :
#CMAB_FULLPALETTE = 0
#CMAF_FULLPALETTE = (1  <<  #CMAB_FULLPALETTE)
#CMAB_NO_INTERMED_UPDATE = 1
#CMAF_NO_INTERMED_UPDATE = (1  <<  #CMAB_NO_INTERMED_UPDATE)
#CMAB_NO_COLOR_LOAD = 2
#CMAF_NO_COLOR_LOAD = (1   <<   #CMAB_NO_COLOR_LOAD)
#CMAB_DUALPF_DISABLE = 3
#CMAF_DUALPF_DISABLE = (1   <<   #CMAB_DUALPF_DISABLE)


Structure PaletteExtra    ;  structure may be extended so watch out!

 pe_Semaphore.SignalSemaphore  ;  shared semaphore for arbitration
 pe_FirstFree.w    ;  *private*
 pe_NFree.w    ;  number of free colors
 pe_FirstShared.w    ;  *private*
 pe_NShared.w    ;  *private*
 *pe_RefCnt.b    ;  *private*
 *pe_AllocList.b    ;  *private*
 *pe_ViewPort.ViewPort   ;  back pointer to viewport
 pe_SharableColors.w   ;  the number of sharable colors.
EndStructure

;  flags values for ObtainPen

#PENB_EXCLUSIVE = 0
#PENB_NO_SETCOLOR = 1

#PENF_EXCLUSIVE = (1   <<  #PENB_EXCLUSIVE)
#PENF_NO_SETCOLOR = (1   <<  #PENB_NO_SETCOLOR)

;  obsolete names for PENF_xxx flags:

#PEN_EXCLUSIVE = #PENF_EXCLUSIVE
#PEN_NO_SETCOLOR = #PENF_NO_SETCOLOR

;  precision values for ObtainBestPen :

#PRECISION_EXACT = -1
#PRECISION_IMAGE = 0
#PRECISION_ICON = 16
#PRECISION_GUI = 32


;  tags for ObtainBestPen:
#OBP_Precision = $84000000
#OBP_FailIfBad = $84000001

;  From V39, MakeVPort() will return an error if there is not enough memory,
;  * or the requested mode cannot be opened with the requested depth with the
;  * given bitmap (for higher bandwidth alignments).
;

#MVP_OK  = 0 ;  you want to see this one
#MVP_NO_MEM = 1 ;  insufficient memory for intermediate workspace
#MVP_NO_VPE = 2 ;  ViewPort does not have a ViewPortExtra, and
;      * insufficient memory to allocate a temporary one.
;
#MVP_NO_DSPINS = 3 ;  insufficient memory for intermidiate copper
;      * instructions.
;
#MVP_NO_DISPLAY = 4 ;  BitMap data is misaligned for this viewport's
;      * mode and depth - see AllocBitMap().
;
#MVP_OFF_BOTTOM = 5 ;  PRIVATE - you will never see this.

;  From V39, MrgCop() will return an error if there is not enough memory,
;  * or for some reason MrgCop() did not need to make any copper lists.
;

#MCOP_OK  = 0 ;  you want to see this one
#MCOP_NO_MEM = 1 ;  insufficient memory to allocate the system
;      * copper lists.
;
#MCOP_NOP = 2 ;  MrgCop() did not merge any copper lists
;      * (eg, no ViewPorts in the list, or all marked as
;      * hidden).
;

Structure DBufInfo
 *dbi_Link1.l
 dbi_Count1.l
 dbi_SafeMessage.Message  ;  replied to when safe to write to old bitmap
 *dbi_UserData1.l   ;  first user data

 *dbi_Link2.l
 dbi_Count2.l
 dbi_DispMessage.Message ;  replied to when new bitmap has been displayed at least
;        once
 *dbi_UserData2.l   ;  second user data
 dbi_MatchLong.l
 *dbi_CopPtr1.l
 *dbi_CopPtr2.l
 *dbi_CopPtr3.l
 dbi_BeamPos1.w
 dbi_BeamPos2.w
EndStructure

