;
; ** $VER: commodities.h 38.4 (24.2.93)
; ** Includes Release 40.15
; **
; ** Commodities definitions
; **
; ** (C) Copyright 1990-1993 Commodore-Amiga Inc.
; ** All Rights Reserved
;

; ***************************************************************************


IncludePath   "PureInclude:"
XIncludeFile "exec/types.pb"
XIncludeFile "exec/ports.pb"
XIncludeFile "devices/inputevent.pb"


; ***************************************************************************


;  object creation macros
;#CxFilter(d)     = CreateCxObj((CX_FILTER.l :     (LONG) d.l :     0).l
;#CxSender(port.l :id)   = CreateCxObG) CX_SEND.l :       (LONG) port.l :  (LONG) id).l
;#CxSignal(task.l :sig)  = CreateCxObG) CX_SIGNAL.l :     (LONG) task.l :  (LONG) sig).l
;#CxTranslate(ie)     = CreateCxObj((CX_TRANSLATE.l :  (LONG) ie.l :    0).l
;#CxDebug(id)     = CreateCxObj((CX_DEBUG.l :      (LONG) id.l :    0).l
;#CxCustom(action.l :id) = CreateCxObG) CX_CUSTOM.l :     (LONG)action.l : (LONG)id).l


; ***************************************************************************


Structure NewBroker
    nb_Version.b   ;  Must be set to NB_VERSION
    pad.b
    *nb_Name.b
    *nb_Title.b
    *nb_Descr.b
    nb_Unique.w
    nb_Flags.w
    nb_Pri.b
    pad2.b
    *nb_Port.MsgPort
    nb_ReservedChannel.w
EndStructure

;  constant for NewBroker.nb_Version
#NB_VERSION = 5     ;  Version of NewBroker structure

;  Sizes for various buffers
#CBD_NAMELEN  = 24
#CBD_TITLELEN = 40
#CBD_DESCRLEN = 40

;  Flags for NewBroker.nb_Unique
#NBU_DUPLICATE = 0
#NBU_UNIQUE    = 1        ;  will not allow duplicates
#NBU_NOTIFY    = 2        ;  sends CXM_UNIQUE to existing broker

;  Flags for NewBroker.nb_Flags
#COF_SHOW_HIDE = 4


; ***************************************************************************


;  Fake data types for system private objects
;typedef CxObj.l
;typedef CxMsg.l

;  Pointer to a function returning a LONG
;typedef (*PFL)().l


; ***************************************************************************


;  Commodities object types
#CX_INVALID = 0     ;  not a valid object (probably null)
#CX_FILTER = 1     ;  input event messages only
#CX_TYPEFILTER = 2     ;  obsolete, do not use
#CX_SEND = 3     ;  sends a message
#CX_SIGNAL = 4     ;  sends a signal
#CX_TRANSLATE = 5     ;  translates input event into chain
#CX_BROKER = 6     ;  application representative
#CX_DEBUG = 7     ;  dumps info to serial port
#CX_CUSTOM = 8     ;  application provides function
#CX_ZERO = 9     ;  system terminator node


; ***************************************************************************


;  Commodities message types
#CXM_IEVENT  = (1  <<  5)
#CXM_COMMAND = (1  <<  6)

;  Only CXM_IEVENT messages are passed through the input network. Other types
;  * of messages are sent to an optional port in your broker. This means that
;  * you must test the message type in your message handling, if input messages
;  * and command messages come to the same port.
;  *
;  * CXM_IEVENT: Messages of this type rattle around the Commodities input
;  *        network. They are sent to you by a Sender object, and passed
;  *        to you as a synchronous function call by a Custom object.
;  *
;  *        The message port or function entry point is stored in the
;  *        object, and the ID field of the message will be set to what
;  *        you arrange issuing object.
;  *
;  *        The data section of the message will point to the input event
;  *        triggering the message.
;  *
;  * CXM_COMMAND: These messages are sent to a port attached to your Broker.
;  *  They are sent to you when the controller program wants your
;  *  program to do something. The ID value identifies the command.
;

;  ID values associated with a message of type CXM_COMMAND
#CXCMD_DISABLE = (15)  ;  please disable yourself
#CXCMD_ENABLE = (17)  ;  please enable yourself
#CXCMD_APPEAR = (19)  ;  open your window, if you can
#CXCMD_DISAPPEAR = (21)  ;  go dormant
#CXCMD_KILL = (23)  ;  go away for good
#CXCMD_LIST_CHG = (27)  ;  Someone changed the broker list
#CXCMD_UNIQUE = (25)  ;  someone tried to create a broker
;           * with your name. Suggest you appear.
;


; ***************************************************************************


Structure InputXpression

    ix_Version.b   ;  must be set to IX_VERSION
    ix_Class.b   ;  class must match exactly

    ix_Code.w   ;  Bits that we want
    ix_CodeMask.w   ;  Set bits here to indicate which bits in ix_Code
;       * are don't care bits.
;
    ix_Qualifier.w   ;  Bits that we want
    ix_QualMask.w   ;  Set bits here to indicate which bits in
;       * ix_Qualifier are don't care bits
;
    ix_QualSame.w   ;  synonyms in qualifier
EndStructure
;typedef IX.InputXpression

;  constant for InputXpression.ix_Version
#IX_VERSION = 2

;  constants for InputXpression.ix_QualSame
#IXSYM_SHIFT = 1 ;  left- and right- shift are equivalent
#IXSYM_CAPS  = 2 ;  either shift or caps lock are equivalent
#IXSYM_ALT   = 4 ;  left- and right- alt are equivalent

#IXSYM_SHIFTMASK = (#IEQUALIFIER_LSHIFT | #IEQUALIFIER_RSHIFT)
#IXSYM_CAPSMASK = (#IXSYM_SHIFTMASK | #IEQUALIFIER_CAPSLOCK)
#IXSYM_ALTMASK = (#IEQUALIFIER_LALT | #IEQUALIFIER_RALT)

;  constant for InputXpression.ix_QualMask
#IX_NORMALQUALS = $7FFF  ;  avoid RELATIVEMOUSE

;  matches nothing
;#NULL_IX(ix)   = ((ix)\ix_Class = IECLASS_NULL)


; ***************************************************************************


;  Error returns from CxBroker()
#CBERR_OK      = 0  ;  No error
#CBERR_SYSERR  = 1  ;  System error, no memory, etc
#CBERR_DUP     = 2  ;  uniqueness violation
#CBERR_VERSION = 3  ;  didn't understand NewBroker.nb_Version


; ***************************************************************************


;  Return values from CxObjError()
#COERR_ISNULL  = 1   ;  you called CxObjError(NULL)
#COERR_NULLATTACH = 2   ;  someone attached NULL to my list
#COERR_BADFILTER  = 4   ;  a bad filter description was given
#COERR_BADTYPE  = 8   ;  unmatched type-specific operation


; ***************************************************************************


